@@ -1,29 +1,28 @@
 package fingerprint
 
 import (
-	"log"
 	"runtime"
 
-	cstructs "github.com/hashicorp/nomad/client/structs"
+	log "github.com/hashicorp/go-hclog"
 	"github.com/shirou/gopsutil/host"
 )
 
 // HostFingerprint is used to fingerprint the host
 type HostFingerprint struct {
 	StaticFingerprinter
-	logger *log.Logger
+	logger log.Logger
 }
 
 // NewHostFingerprint is used to create a Host fingerprint
-func NewHostFingerprint(logger *log.Logger) Fingerprint {
-	f := &HostFingerprint{logger: logger}
+func NewHostFingerprint(logger log.Logger) Fingerprint {
+	f := &HostFingerprint{logger: logger.Named("host")}
 	return f
 }
 
-func (f *HostFingerprint) Fingerprint(req *cstructs.FingerprintRequest, resp *cstructs.FingerprintResponse) error {
+func (f *HostFingerprint) Fingerprint(req *FingerprintRequest, resp *FingerprintResponse) error {
 	hostInfo, err := host.Info()
 	if err != nil {
-		f.logger.Println("[WARN] Error retrieving host information: ", err)
+		f.logger.Warn("error retrieving host information", "error", err)
 		return err
 	}
 
