@@ -1,6 +1,7 @@
 package agent
 
 import (
+	"fmt"
 	"io/ioutil"
 	"os"
 	"path/filepath"
@@ -95,6 +96,7 @@ func TestAgent_ServerConfig(t *testing.T) {
 	require.NoError(t, conf.normalizeAddrs())
 
 	out, err = a.serverConfig()
+	require.NoError(t, err)
 	require.Equal(t, "127.0.0.2", out.RPCAddr.IP.String())
 	require.Equal(t, 4003, out.RPCAddr.Port)
 	require.Equal(t, "127.0.0.2", out.SerfConfig.MemberlistConfig.BindAddr)
@@ -166,14 +168,12 @@ func TestAgent_ServerConfig(t *testing.T) {
 	conf.Server.BootstrapExpect = 1
 	out, err = a.serverConfig()
 	require.NoError(t, err)
-	require.True(t, out.Bootstrap)
-	require.Equal(t, int32(0), out.BootstrapExpect)
+	require.Equal(t, 1, out.BootstrapExpect)
 
 	conf.Server.BootstrapExpect = 3
 	out, err = a.serverConfig()
 	require.NoError(t, err)
-	require.False(t, out.Bootstrap)
-	require.Equal(t, int32(3), out.BootstrapExpect)
+	require.Equal(t, 3, out.BootstrapExpect)
 }
 
 func TestAgent_ServerConfig_SchedulerFlags(t *testing.T) {
@@ -186,6 +186,7 @@ func TestAgent_ServerConfig_SchedulerFlags(t *testing.T) {
 			"default case",
 			nil,
 			structs.SchedulerConfiguration{
+				SchedulerAlgorithm: "binpack",
 				PreemptionConfig: structs.PreemptionConfig{
 					SystemSchedulerEnabled: true,
 				},
@@ -363,6 +364,113 @@ func TestAgent_ServerConfig_Limits_OK(t *testing.T) {
 	}
 }
 
+func TestAgent_ServerConfig_RaftMultiplier_Ok(t *testing.T) {
+	t.Parallel()
+
+	cases := []struct {
+		multiplier         *int
+		electionTimout     time.Duration
+		heartbeatTimeout   time.Duration
+		leaderLeaseTimeout time.Duration
+		commitTimeout      time.Duration
+	}{
+		// nil, 0 are the defaults of the Raft library.
+		// Expected values are hardcoded to detect changes from raft.
+		{
+			multiplier: nil,
+
+			electionTimout:     1 * time.Second,
+			heartbeatTimeout:   1 * time.Second,
+			leaderLeaseTimeout: 500 * time.Millisecond,
+			commitTimeout:      50 * time.Millisecond,
+		},
+
+		{
+			multiplier: helper.IntToPtr(0),
+
+			electionTimout:     1 * time.Second,
+			heartbeatTimeout:   1 * time.Second,
+			leaderLeaseTimeout: 500 * time.Millisecond,
+			commitTimeout:      50 * time.Millisecond,
+		},
+		{
+			multiplier: helper.IntToPtr(1),
+
+			electionTimout:     1 * time.Second,
+			heartbeatTimeout:   1 * time.Second,
+			leaderLeaseTimeout: 500 * time.Millisecond,
+			commitTimeout:      50 * time.Millisecond,
+		},
+		{
+			multiplier: helper.IntToPtr(5),
+
+			electionTimout:     5 * time.Second,
+			heartbeatTimeout:   5 * time.Second,
+			leaderLeaseTimeout: 2500 * time.Millisecond,
+			commitTimeout:      250 * time.Millisecond,
+		},
+		{
+			multiplier: helper.IntToPtr(6),
+
+			electionTimout:     6 * time.Second,
+			heartbeatTimeout:   6 * time.Second,
+			leaderLeaseTimeout: 3000 * time.Millisecond,
+			commitTimeout:      300 * time.Millisecond,
+		},
+		{
+			multiplier: helper.IntToPtr(10),
+
+			electionTimout:     10 * time.Second,
+			heartbeatTimeout:   10 * time.Second,
+			leaderLeaseTimeout: 5000 * time.Millisecond,
+			commitTimeout:      500 * time.Millisecond,
+		},
+	}
+
+	for _, tc := range cases {
+		v := "default"
+		if tc.multiplier != nil {
+			v = fmt.Sprintf("%v", *tc.multiplier)
+		}
+		t.Run(v, func(t *testing.T) {
+			conf := DevConfig(nil)
+			require.NoError(t, conf.normalizeAddrs())
+
+			conf.Server.RaftMultiplier = tc.multiplier
+
+			serverConf, err := convertServerConfig(conf)
+			require.NoError(t, err)
+
+			assert.Equal(t, tc.electionTimout, serverConf.RaftConfig.ElectionTimeout, "election timeout")
+			assert.Equal(t, tc.heartbeatTimeout, serverConf.RaftConfig.HeartbeatTimeout, "heartbeat timeout")
+			assert.Equal(t, tc.leaderLeaseTimeout, serverConf.RaftConfig.LeaderLeaseTimeout, "leader lease timeout")
+			assert.Equal(t, tc.commitTimeout, serverConf.RaftConfig.CommitTimeout, "commit timeout")
+		})
+	}
+}
+
+func TestAgent_ServerConfig_RaftMultiplier_Bad(t *testing.T) {
+	t.Parallel()
+
+	cases := []int{
+		-1,
+		100,
+	}
+
+	for _, tc := range cases {
+		t.Run(fmt.Sprintf("%v", tc), func(t *testing.T) {
+			conf := DevConfig(nil)
+			require.NoError(t, conf.normalizeAddrs())
+
+			conf.Server.RaftMultiplier = &tc
+
+			_, err := convertServerConfig(conf)
+			require.Error(t, err)
+			require.Contains(t, err.Error(), "raft_multiplier cannot be")
+		})
+	}
+}
+
 func TestAgent_ClientConfig(t *testing.T) {
 	t.Parallel()
 	conf := DefaultConfig()
@@ -659,7 +767,8 @@ func TestServer_Reload_TLS_Certificate(t *testing.T) {
 	}
 
 	agent := &Agent{
-		config: agentConfig,
+		auditor: &noOpAuditor{},
+		config:  agentConfig,
 	}
 
 	newConfig := &Config{
@@ -707,7 +816,8 @@ func TestServer_Reload_TLS_Certificate_Invalid(t *testing.T) {
 	}
 
 	agent := &Agent{
-		config: agentConfig,
+		auditor: &noOpAuditor{},
+		config:  agentConfig,
 	}
 
 	newConfig := &Config{
@@ -786,8 +896,9 @@ func TestServer_Reload_TLS_UpgradeToTLS(t *testing.T) {
 	}
 
 	agent := &Agent{
-		logger: logger,
-		config: agentConfig,
+		auditor: &noOpAuditor{},
+		logger:  logger,
+		config:  agentConfig,
 	}
 
 	newConfig := &Config{
@@ -835,8 +946,9 @@ func TestServer_Reload_TLS_DowngradeFromTLS(t *testing.T) {
 	}
 
 	agent := &Agent{
-		logger: logger,
-		config: agentConfig,
+		logger:  logger,
+		config:  agentConfig,
+		auditor: &noOpAuditor{},
 	}
 
 	newConfig := &Config{
