@@ -3,8 +3,8 @@
 package docker
 
 import (
+	"github.com/docker/docker/oci/caps"
 	docker "github.com/fsouza/go-dockerclient"
-	"github.com/moby/moby/daemon/caps"
 )
 
 func getPortBinding(ip string, port string) []docker.PortBinding {
@@ -19,7 +19,7 @@ func tweakCapabilities(basics, adds, drops []string) ([]string, error) {
 		basics[i] = "CAP_" + cap
 	}
 
-	effectiveCaps, err := caps.TweakCapabilities(basics, adds, drops)
+	effectiveCaps, err := caps.TweakCapabilities(basics, adds, drops, nil, false)
 	if err != nil {
 		return effectiveCaps, err
 	}
