@@ -1,9 +1,9 @@
 import Controller from '@ember/controller';
 
-export default Controller.extend({
+export default class RunController extends Controller {
   onSubmit(id, namespace) {
     this.transitionToRoute('jobs.job', id, {
       queryParams: { jobNamespace: namespace },
     });
-  },
-});
+  }
+}
